@@ -23,18 +23,15 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-#include <QLabel>
-#include <QLayout>
+#include "kcmlocale.h"
+
 #include <QPushButton>
 
 #include <KDebug>
-#include <KDialog>
-#include <KIconLoader>
 #include <KConfig>
 #include <KConfigGroup>
 #include <KStandardDirs>
 
-#include "kcmlocale.h"
 #include "kcmlocale.moc"
 #include "toplevel.h"
 #include "klanguagebutton.h"
