@@ -1,7 +1,7 @@
 /*
     This file is part of the Mollet network library, part of the KDE project.
 
-    Copyright 2009 Friedrich W. H. Kossebau <kossebau@kde.org>
+    Copyright 2009-2010 Friedrich W. H. Kossebau <kossebau@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,8 @@
 
 // lib
 #include "netservice_p.h"
+// #include "service.h"
+#include "upnp/cagibidevice.h"
 // KDE
 #include <KUrl>
 
@@ -48,13 +50,13 @@ static const DNSSDServiceDatum DNSSDServiceData[] =
 {
     // file services
     { "_ftp._tcp",         "ftp",        "folder-remote",  true, "ftp",     "path", "u", "p" },
+    { "_sftp-ssh._tcp",    "sftp-ssh",   "folder-remote",  true, "sftp",    0,      "u", "p" },
+    { "_ftps._tcp",        "ftps",       "folder-remote",  true, "ftps",    "path", "u", "p" },
     { "_nfs._tcp",         "nfs",        "folder-remote",  true, "nfs",     "path", 0, 0 },
     { "_afpovertcp._tcp",  "afpovertcp", "folder-remote",  true, "afp",     "path", 0, 0 },
     { "_smb._tcp",         "smb",        "folder-remote",  true, "smb",     "path", "u", "p" },
     { "_webdav._tcp",      "webdav",     "folder-remote",  true, "webdav",  "path", "u", "p" },
     { "_webdavs._tcp",     "webdavs",    "folder-remote",  true, "webdavs", "path", "u", "p" },
-    { "_sftp-ssh._tcp",    "sftp-ssh",   "folder-remote",  true, "sftp",    0,      "u", "p" },
-    { "_ftps._tcp",        "ftps",       "folder-remote",  true, "ftps",    "path", "u", "p" },
 
     { "_svn._tcp",    "svn",   "folder-sync",  true, 0, 0, 0, 0 },
     { "_rsync._tcp",  "rsync", "folder-sync",  true, 0, 0, 0, 0 },
@@ -63,43 +65,36 @@ static const DNSSDServiceDatum DNSSDServiceData[] =
     { "_imap._tcp",   "imap",   "email",  false, 0, 0, 0, 0 },
     { "_pop3._tcp",   "pop3",   "email",  false, "pop3", 0, 0, 0 },
 
-    //  shell services
+    // shell services
     { "_ssh._tcp",    "ssh",    "terminal",  false, "ssh",    0,      "u", "p" },
     { "_telnet._tcp", "telnet", "terminal",  false, "telnet", 0,      "u", "p" },
     { "_rfb._tcp",    "rfb",    "krfb",      false, "vnc",    "path", "u", "p" },
     { "_rdp._tcp",    "rdp",    "krfb",      false, "rdp", 0, 0, 0 },
 
     // other standard services
-    { "_ipp._tcp",    "ipp",    "printer",                false, "ipp",  "path", "u", "p" },
     { "_http._tcp",   "http",   "folder-html",            false, "http", "path", "u", "p" },
     { "_ntp._udp",    "ntp",    "xclock",                 false, 0, 0, 0, 0 },
     { "_ldap._tcp",   "ldap",   "user-group-properties",  false, "ldap", 0, 0, 0 },
-    { "_pgpkey-ldap._tcp",  "pgpkey-hkp",   "application-pgp-keys",  false, 0, 0, 0, 0 },
-    { "_pgpkey-ldap._tcp",  "pgpkey-hkp",   "application-pgp-keys",  false, 0, 0, 0, 0 },
-    { "_pgpkey-hkp._tcp",   "pgpkey-hkp",   "application-pgp-keys",  false, 0, 0, 0, 0 },
-    { "_pgpkey-https._tcp", "pgpkey-https", "application-pgp-keys",  true, "https", "path", 0, 0 },
-    { "_presence._tcp",     "presence",     "im-user",               false, "presence", 0, 0, 0 },
 
-    // Apple
-    { "_airport._tcp",       "airport",       "network-wireless",  false, 0, 0, 0, 0 },
-    { "_daap._tcp",          "daap",          "folder-sound",      false, 0, 0, 0, 0 },
-    { "_eppc._tcp",          "eppc",          "network-connect",   false, 0, 0, 0, 0 },
-    { "_net-assistant._udp", "net-assistant", "services",          false, 0, 0, 0, 0 },
-    { "_odisk._tcp",         "odisk",         "media-optical",     false, 0, 0, 0, 0 },
-    { "_raop._tcp",          "raop",          "speaker",           false, 0, 0, 0, 0 },
-    { "_touch-able._tcp",    "touch-able",    "input-tablet",      false, 0, 0, 0, 0 },
-    { "_workstation._tcp",   "workstation",   "network-workgroup", false, 0, 0, 0, 0 },
-    { "_sleep-proxy._udp",   "sleep-proxy",   "services",          false, 0, 0, 0, 0 },
+    // user2user (chat, collaboration)
+    { "_xmpp-server._tcp", "xmpp-server", "xchat",         false, "jabber", 0, 0, 0 },
+    { "_presence._tcp",    "presence",    "im-user",       false, "presence", 0, 0, 0 },
+    { "_lobby._tcp",       "lobby",       "document-edit", false, 0, 0, 0, 0 },
+    { "_giver._tcp",       "giver",       "folder-remote", false, 0, 0, 0, 0 },
+
+    // printing
+    { "_ipp._tcp",            "ipp",            "printer", false, "ipp",  "path", "u", "p" },
+    { "_printer._tcp",        "printer",        "printer", false, 0, 0, 0, 0 },
+    { "_pdl-datastream._tcp", "pdl-datastream", "printer", false, 0, 0, 0, 0 },
 
     // KDE workspace
     { "_plasma._tcp",       "plasma",      "plasma",       false, "plasma", 0, 0, 0 },
 
     // KDE games
-    // TODO: http as protocol is a fake, but the KIO system currently only supports known protocols, so we pretend it is http
     { "_kbattleship._tcp",  "kbattleship", "kbattleship",  false, "kbattleship", 0, 0, 0 },
-    { "_lskat._tcp",        "lskat",       "lskat",        false, "http", 0, 0, 0 },
-    { "_kfourinline._tcp",  "kfourinline", "kfourinline",  false, "http", 0, 0, 0 },
-    { "_ksirk._tcp",        "ksirk",       "ksirk",        false, "http", 0, 0, 0 },
+    { "_lskat._tcp",        "lskat",       "lskat",        false, 0, 0, 0, 0 },
+    { "_kfourinline._tcp",  "kfourinline", "kfourinline",  false, 0, 0, 0, 0 },
+    { "_ksirk._tcp",        "ksirk",       "ksirk",        false, 0, 0, 0, 0 },
 
     // hardware
     { "_pulse-server._tcp","pulse-server","audio-card",          false, 0, 0, 0, 0 },
@@ -109,11 +104,31 @@ static const DNSSDServiceDatum DNSSDServiceData[] =
     { "_airmouse._tcp",    "airmouse",    "input-mouse",         false, 0, 0, 0, 0 },
 
     // else
-    { "_xmpp-server._tcp", "xmpp-server", "xchat",               false, "jabber", 0, 0, 0 },
-    { "_lobby._tcp",       "lobby",       "document-edit",       false, 0, 0, 0, 0 },
-    { "_giver._tcp",       "giver",       "folder-remote",       false, 0, 0, 0, 0 },
+    { "_postgresql._tcp",  "postgresql",  "server-database",     false, 0, 0, 0, 0 },
     { "_realplayfavs._tcp","realplayfavs","favorites",           false, 0, 0, 0, 0 },
-    { "_ggz._tcp",         "ggz",         "applications-games",  false, "ggz", 0, 0, 0 }
+    { "_acrobatSRV._tcp",  "acrobat-server","application-pdf",   false, 0, 0, 0, 0 },
+    { "_adobe-vc._tcp",    "adobe-vc",    "services",   false, 0, 0, 0, 0 },
+    { "_ggz._tcp",         "ggz",         "applications-games",  false, "ggz", 0, 0, 0 },
+
+    { "_pgpkey-ldap._tcp",  "pgpkey-ldap",  "application-pgp-keys",  false, 0, 0, 0, 0 },
+    { "_pgpkey-hkp._tcp",   "pgpkey-hkp",   "application-pgp-keys",  false, 0, 0, 0, 0 },
+    { "_pgpkey-https._tcp", "pgpkey-https", "application-pgp-keys",  true, "https", "path", 0, 0 },
+
+    // Apple
+    { "_airport._tcp",       "airport",       "network-wireless",  false, 0, 0, 0, 0 },
+    { "_daap._tcp",          "daap",          "folder-sound",      false, 0, 0, 0, 0 },
+    { "_dacp._tcp",          "dacp",          "folder-sound",      false, 0, 0, 0, 0 },
+    { "_eppc._tcp",          "eppc",          "network-connect",   false, 0, 0, 0, 0 },
+    { "_net-assistant._udp", "net-assistant", "services",          false, 0, 0, 0, 0 },
+    { "_odisk._tcp",         "odisk",         "media-optical",     false, 0, 0, 0, 0 },
+    { "_raop._tcp",          "raop",          "speaker",           false, 0, 0, 0, 0 },
+    { "_touch-able._tcp",    "touch-able",    "input-tablet",      false, 0, 0, 0, 0 },
+    { "_workstation._tcp",   "workstation",   "network-workgroup", false, 0, 0, 0, 0 },
+    { "_sleep-proxy._udp",   "sleep-proxy",   "services",          false, 0, 0, 0, 0 },
+    { "_nssocketport._tcp",  "nssocketport",  "services",          false, 0, 0, 0, 0 },
+    { "_home-sharing._tcp",  "home-sharing",  "services",          false, 0, 0, 0, 0 },
+    { "_appletv-itunes._tcp","appletv-itunes","services",          false, 0, 0, 0, 0 },
+    { "_appletv-pair._tcp",  "appletv-pair",  "services",          false, 0, 0, 0, 0 }
 };
 //     result["_ssh._tcp"]=      DNSSDNetServiceBuilder(i18n("Remote disk (fish)"),     "fish",   "service/ftp", QString(), "u", "p");
 // network-server-database icon
@@ -191,6 +206,29 @@ NetServicePrivate* SimpleItemFactory::createNetService( const DNSSD::RemoteServi
     return result;
 }
 
+bool SimpleItemFactory::canCreateNetSystemFromUpnp( const Cagibi::Device& upnpDevice ) const
+{
+Q_UNUSED( upnpDevice )
+    return true;
+}
+// TODO: add KIcon with specialiced KIconLoader (fetches Icons via D-Bus)
+NetServicePrivate* SimpleItemFactory::createNetService( const Cagibi::Device& upnpDevice, const NetDevice& device ) const
+{
+    NetServicePrivate* result;
+
+    QString url = upnpDevice.presentationUrl();
+    if( url.isEmpty() )
+    {
+        url = QString::fromLatin1( "upnp://" );
+        url.append( upnpDevice.udn() );
+    }
+    result = new NetServicePrivate( upnpDevice.friendlyName(),
+        QString::fromLatin1("unknown"),
+        "upnp."+upnpDevice.type(), device, url );
+
+    return result;
+}
+
 SimpleItemFactory::~SimpleItemFactory() {}
 
 }
