@@ -22,11 +22,10 @@
 
 #include "fixhosturifilter.h"
 
+#include <QtCore/QTimer>
+
 #include <kdebug.h>
 #include <kurl.h>
-#include <k3resolver.h>
-
-using namespace KNetwork;
 
 /**
  * IMPORTANT: If you change anything here, please run the regression test
@@ -34,7 +33,7 @@ using namespace KNetwork;
  */
  
 FixHostUriFilter::FixHostUriFilter( QObject *parent, const QVariantList & /*args*/ )
-    : KUriFilterPlugin( "fixhosturifilter", parent )
+    : KUriFilterPlugin( "fixhosturifilter", parent ), m_hostExists( false )
 {
 }
 
@@ -42,8 +41,8 @@ bool FixHostUriFilter::filterUri( KUriFilterData& data ) const
 {
     KUrl url = data.uri();
     QString cmd = url.url();
-    
-    kDebug() << "FixHostUriFilter::filterUri: " << url;
+
+    kDebug(7023) <<  url;
 
     KUrl url2 = url;
     url2.setHost( "www." + url.host());
@@ -53,19 +52,46 @@ bool FixHostUriFilter::filterUri( KUriFilterData& data ) const
     {
         setFilteredUri( data, url2 );
         setUriType( data, KUriFilterData::NetProtocol );
-        
-        kDebug() << "FilteredUri: " << data.uri();
+
+        kDebug() << "filtered to" << data.uri();
         return true;
     }
 
     return false;
 }
 
-bool FixHostUriFilter::exists( const KUrl& url )
+void FixHostUriFilter::lookedUp( const QHostInfo &hostInfo )
 {
-    KResolver resolver( url.host());
-    resolver.setFamily( KResolver::InetFamily );
-    return( resolver.start() && resolver.wait( 1000 ) && resolver.error() == KResolver::NoError );
+    if (hostInfo.lookupId() == m_lookupId) {
+        m_hostExists = ( hostInfo.error() == QHostInfo::NoError );
+        if (m_eventLoop) {
+            m_eventLoop->exit();
+        }
+    }
+}
+
+bool FixHostUriFilter::exists( const KUrl& url ) const
+{
+    QEventLoop eventLoop;
+
+    m_hostExists = false;
+    m_eventLoop = &eventLoop;
+
+    FixHostUriFilter *self = const_cast<FixHostUriFilter*>( this );
+    m_lookupId = QHostInfo::lookupHost( url.host(), self, SLOT(lookedUp(QHostInfo)) );
+
+    QTimer t;
+    connect( &t, SIGNAL(timeout()), &eventLoop, SLOT(quit()) );
+    t.start(1000);
+
+    eventLoop.exec();
+    m_eventLoop = 0;
+
+    t.stop();
+
+    QHostInfo::abortHostLookup( m_lookupId );
+
+    return m_hostExists;
 }
 
 K_PLUGIN_FACTORY(FixHostUriFilterFactory, registerPlugin<FixHostUriFilter>();)
