@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2007-2009 Sebastian Trueg <trueg@kde.org>
+  Copyright (C) 2007-2010 Sebastian Trueg <trueg@kde.org>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
@@ -21,10 +21,10 @@
 #include "util.h"
 #include "nfo.h"
 #include "nie.h"
+#include "nrl.h"
 
 #include <Soprano/Soprano>
 #include <Soprano/Vocabulary/RDF>
-#include <Soprano/Vocabulary/Xesam>
 #include <Soprano/LiteralValue>
 #include <Soprano/Node>
 #include <Soprano/QueryResultIterator>
@@ -128,11 +128,14 @@ namespace {
     class FileMetaData
     {
     public:
-        FileMetaData( const KUrl& url );
+        FileMetaData( const Strigi::AnalysisResult* idx );
 
         /// stores basic data including the nie:url and the nrl:GraphMetadata in \p model
         void storeBasicData( Soprano::Model* model );
 
+        /// map a blank node to a resource
+        QUrl mapNode( const std::string& s );
+
         /// The resource URI
         QUrl resourceUri;
 
@@ -148,8 +151,12 @@ namespace {
         /// a buffer for all plain-text content generated by strigi
         std::string content;
 
+    private:
+        /// The Strigi result
+        const Strigi::AnalysisResult* m_analysisResult;
+
         /// mapping from blank nodes used in addTriplet to our urns
-        QMap<std::string, QUrl> blankNodeMap;
+        QMap<std::string, QUrl> m_blankNodeMap;
     };
 
     class RegisteredFieldData
@@ -171,10 +178,12 @@ namespace {
         bool isRdfType;
     };
 
-    FileMetaData::FileMetaData( const KUrl& url )
-        : fileUrl( url ),
-          fileInfo( url.toLocalFile() )
+    FileMetaData::FileMetaData( const Strigi::AnalysisResult* idx )
+        : m_analysisResult( idx )
     {
+        fileUrl = createFileUrl( idx );
+        fileInfo = fileUrl.toLocalFile();
+
         // determine the resource URI by using Nepomuk::Resource's power
         // this will automatically find previous uses of the file in question
         // with backwards compatibility
@@ -184,6 +193,27 @@ namespace {
         context = Nepomuk::ResourceManager::instance()->generateUniqueUri( "ctx" );
     }
 
+    QUrl FileMetaData::mapNode( const std::string& s )
+    {
+        if ( s[0] == ':' ) {
+            if( m_blankNodeMap.contains( s ) ) {
+                return m_blankNodeMap[s];
+            }
+            else {
+                QUrl urn = Nepomuk::ResourceManager::instance()->generateUniqueUri( QString() );
+                m_blankNodeMap.insert( s, urn );
+                return urn;
+            }
+        }
+        // special case to properly handle nie:isPartOf relations created for containers
+        else if ( s == m_analysisResult->path() ) {
+            return resourceUri;
+        }
+        else {
+            return QUrl::fromEncoded( s.c_str() );
+        }
+    }
+
     void FileMetaData::storeBasicData( Soprano::Model* model )
     {
         model->addStatement( resourceUri, Nepomuk::Vocabulary::NIE::url(), fileUrl, context );
@@ -207,7 +237,7 @@ namespace {
         QUrl metaDataContext = Nepomuk::ResourceManager::instance()->generateUniqueUri( "ctx" );
         model->addStatement( context,
                              Vocabulary::RDF::type(),
-                             Vocabulary::NRL::InstanceBase(),
+                             Nepomuk::Vocabulary::NRL::DiscardableInstanceBase(),
                              metaDataContext );
         model->addStatement( context,
                              Vocabulary::NAO::created(),
@@ -219,10 +249,10 @@ namespace {
                              metaDataContext );
         model->addStatement( metaDataContext,
                              Vocabulary::RDF::type(),
-                             Vocabulary::NRL::GraphMetadata(),
+                             Nepomuk::Vocabulary::NRL::GraphMetadata(),
                              metaDataContext );
         model->addStatement( metaDataContext,
-                             Vocabulary::NRL::coreGraphMetadataFor(),
+                             Nepomuk::Vocabulary::NRL::coreGraphMetadataFor(),
                              context,
                              metaDataContext );
     }
@@ -273,22 +303,6 @@ public:
         }
     }
 
-    QUrl mapNode( FileMetaData* fmd, const std::string& s ) {
-        if ( s[0] == ':' ) {
-            if( fmd->blankNodeMap.contains( s ) ) {
-                return fmd->blankNodeMap[s];
-            }
-            else {
-                QUrl urn = Nepomuk::ResourceManager::instance()->generateUniqueUri( QString() );
-                fmd->blankNodeMap.insert( s, urn );
-                return urn;
-            }
-        }
-        else {
-            return QUrl::fromEncoded( s.c_str() );
-        }
-    }
-
     Soprano::Model* repository;
 
     //
@@ -332,7 +346,7 @@ void Strigi::NepomukIndexWriter::deleteEntries( const std::vector<std::string>&
 {
     for ( unsigned int i = 0; i < entries.size(); ++i ) {
         QString path = QString::fromUtf8( entries[i].c_str() );
-        removeIndexedData( path );
+        removeIndexedData( KUrl(), path );
     }
 }
 
@@ -356,10 +370,10 @@ void Strigi::NepomukIndexWriter::startAnalysis( const AnalysisResult* idx )
     }
 
     // create the file data used during the analysis
-    FileMetaData* data = new FileMetaData( createFileUrl( idx ) );
+    FileMetaData* data = new FileMetaData( idx );
 
     // remove previously indexed data
-    removeIndexedData( data->resourceUri, true );
+    removeIndexedData( data->resourceUri, data->fileUrl );
 
     // It is important to keep the resource URI between updates (especially for sharing of files)
     // However, when updating data from pre-KDE 4.4 times we want to get rid of old file:/ resource
@@ -450,7 +464,7 @@ void Strigi::NepomukIndexWriter::addValue( const AnalysisResult* idx,
             if ( value[0] == ':' ) {
                 Nepomuk::Types::Property property( rfd->property );
                 if ( property.range().isValid() ) {
-                    statement.setObject( d->mapNode( md, value ) );
+                    statement.setObject( md->mapNode( value ) );
                 }
             }
         }
@@ -546,11 +560,11 @@ void Strigi::NepomukIndexWriter::addTriplet( const std::string& s,
 
     FileMetaData* md = fileDataForResult( d->currentResultStack.top() );
 
-    QUrl subject = d->mapNode( md, s );
-    Nepomuk::Types::Property property( d->mapNode( md, p ) );
+    QUrl subject = md->mapNode( s );
+    Nepomuk::Types::Property property( md->mapNode( p ) );
     Soprano::Node object;
     if ( property.range().isValid() )
-        object = d->mapNode( md, o );
+        object = md->mapNode( o );
     else
         object = Soprano::LiteralValue::fromString( QString::fromUtf8( o.c_str() ), property.literalRangeType().dataTypeUri() );
 
@@ -610,34 +624,36 @@ void Strigi::NepomukIndexWriter::releaseWriterData( const Strigi::FieldRegister&
 }
 
 
-void Strigi::NepomukIndexWriter::removeIndexedData( const KUrl& url, bool isResourceUri )
+void Strigi::NepomukIndexWriter::removeIndexedData( const KUrl& uri, const KUrl& url )
 {
 //    kDebug() << url;
 
     if ( url.isEmpty() )
         return;
 
-    //
-    // We are compatible with old Xesam data where the url was encoded as a string instead of a url,
-    // thus the weird query
-    //
     QString query;
-    if ( isResourceUri ) {
+    if ( uri == url || url.isEmpty() ) {
          query = QString::fromLatin1( "select ?g where { ?g %1 %2 . }" )
                     .arg( Soprano::Node::resourceToN3( Strigi::Ontology::indexGraphFor() ),
+                          Soprano::Node::resourceToN3( uri ) );
+    }
+    else if ( !uri.isEmpty() ) {
+        // makes sure we also catch data created by buggy versions of ourselves.
+        query = QString::fromLatin1( "select ?g where { { ?g %1 %2 . } UNION { ?g %1 %3 . } }" )
+                    .arg( Soprano::Node::resourceToN3( Strigi::Ontology::indexGraphFor() ),
+                          Soprano::Node::resourceToN3( uri ),
                           Soprano::Node::resourceToN3( url ) );
     }
     else {
+        // The last UNION makes sure we also catch data created by buggy versions of ourselves.
         query = QString::fromLatin1( "select ?g where { "
-                                     "{ ?r %3 %1 . } "
+                                     "{ "
+                                     "?r %2 %1 . "
+                                     "?g %3 ?r . } "
                                      "UNION "
-                                     "{ ?r %3 %2 . } "
-                                     "UNION "
-                                     "{ ?r %4 %2 . } . "
-                                     "?g %5 ?r . }" )
-                    .arg( Node::literalToN3( url.path() ),
-                          Node::resourceToN3( url ),
-                          Node::resourceToN3( Vocabulary::Xesam::url() ),
+                                     "{ ?g %3 %1 . }"
+                                     "}")
+                    .arg( Node::resourceToN3( url ),
                           Node::resourceToN3( Nepomuk::Vocabulary::NIE::url() ),
                           Node::resourceToN3( Strigi::Ontology::indexGraphFor() ) );
     }
