@@ -52,12 +52,14 @@ class AppletInterface : public QObject
     Q_ENUMS(AspectRatioMode)
     Q_ENUMS(BackgroundHints)
     Q_ENUMS(QtOrientation)
+    Q_ENUMS(QtModifiers)
     Q_ENUMS(QtAnchorPoint)
     Q_ENUMS(QtCorner)
     Q_ENUMS(QtSizePolicy)
     Q_ENUMS(QtAlignment)
     Q_ENUMS(QtMouseButton)
     Q_ENUMS(AnimationDirection)
+    Q_ENUMS(IntervalAlignment)
     Q_PROPERTY(AspectRatioMode aspectRatioMode READ aspectRatioMode WRITE setAspectRatioMode)
     Q_PROPERTY(FormFactor formFactor READ formFactor)
     Q_PROPERTY(Location location READ location)
@@ -68,6 +70,7 @@ class AppletInterface : public QObject
     Q_PROPERTY(BackgroundHints backgroundHints WRITE setBackgroundHints READ backgroundHints)
     Q_PROPERTY(QGraphicsLayout *layout WRITE setLayout READ layout)
     Q_PROPERTY(bool immutable READ immutable)
+    Q_PROPERTY(bool userConfiguring READ userConfiguring) // @since 4.5
     Q_PROPERTY(int apiVersion READ apiVersion)
     Q_PROPERTY(QRectF rect READ rect)
     Q_PROPERTY(QSizeF size READ size)
@@ -123,6 +126,14 @@ enum AspectRatioMode {
 };
 
 //From Qt namespace
+enum QtModifiers {
+    QtNoModifier = Qt::NoModifier,
+    QtShiftModifier = Qt::ShiftModifier,
+    QtControlModifier = Qt::ControlModifier,
+    QtAltModifier = Qt::AltModifier,
+    QtMetaModifier = Qt::MetaModifier
+};
+
 enum QtOrientation {
     QtHorizontal= Qt::Horizontal,
     QtVertical = Qt::Vertical
@@ -190,6 +201,12 @@ enum AnimationDirection {
     AnimationForward = QAbstractAnimation::Forward,
     AnimationBackward = QAbstractAnimation::Backward
 };
+
+enum IntervalAlignment {
+    NoAlignment = 0,
+    AlignToMinute,
+    AlignToHour
+};
 //-------------------------------------------------------------------
 
     Q_INVOKABLE void gc();
@@ -202,7 +219,6 @@ enum AnimationDirection {
     Q_INVOKABLE AspectRatioMode aspectRatioMode() const;
     Q_INVOKABLE void setAspectRatioMode(AspectRatioMode mode);
 
-
     Q_INVOKABLE void setFailedToLaunch(bool failed, const QString &reason = QString());
 
     Q_INVOKABLE bool isBusy() const;
@@ -243,10 +259,14 @@ enum AnimationDirection {
     Q_INVOKABLE bool include(const QString &script);
     Q_INVOKABLE bool hasExtension(const QString &extension) const;
 
+    Q_INVOKABLE void debug(const QString &msg);
     Q_INVOKABLE QObject *findChild(const QString &name) const;
 
     Q_INVOKABLE Plasma::Extender *extender() const;
 
+    Q_INVOKABLE void addEventListener(const QString &event, const QScriptValue &func);
+    Q_INVOKABLE void removeEventListener(const QString &event, const QScriptValue &func);
+
     Plasma::DataEngine *dataEngine(const QString &name);
 
     const Plasma::Package *package() const;
@@ -254,11 +274,9 @@ enum AnimationDirection {
     QGraphicsLayout *layout() const;
     void setLayout(QGraphicsLayout *);
     bool immutable() const;
+    bool userConfiguring() const;
     int apiVersion() const;
 
-    SimpleJavaScriptApplet::AllowedUrls allowedUrls() const;
-    void setAllowedUrls(const SimpleJavaScriptApplet::AllowedUrls &allowedUrls);
-
     inline Plasma::Applet *applet() const { return m_appletScriptEngine->applet(); }
 
 Q_SIGNALS:
@@ -276,13 +294,14 @@ private:
     QSignalMapper *m_actionSignals;
     QString m_currentConfig;
     QMap<QString, Plasma::ConfigLoader*> m_configs;
-    SimpleJavaScriptApplet::AllowedUrls m_allowedUrls;
 };
 
 class PopupAppletInterface : public AppletInterface
 {
     Q_OBJECT
     Q_PROPERTY(QIcon popupIcon READ popupIcon WRITE setPopupIcon)
+    Q_PROPERTY(bool passivePopup READ isPassivePopup WRITE setPassivePopup)
+    Q_PROPERTY(QGraphicsWidget *popupWidget READ popupWidget WRITE setPopupWidget)
 
 public:
     PopupAppletInterface(SimpleJavaScriptApplet *parent);
@@ -292,8 +311,17 @@ public:
 
     inline Plasma::PopupApplet *popupApplet() const { return static_cast<Plasma::PopupApplet *>(m_appletScriptEngine->applet()); }
 
+    void setPassivePopup(bool passive);
+    bool isPassivePopup() const;
+
+    void setPopupWidget(QGraphicsWidget *widget);
+    QGraphicsWidget *popupWidget();
+
 public Q_SLOTS:
     void setPopupIconByName(const QString &name);
+    void togglePopup();
+    void hidePopup();
+    void showPopup();
 };
 
 #endif
